#
#  $Id$
#
#  (C) Copyright 2003 Kai Sterker <kaisterker@linuxgames.com>
#  Part of the Adonthell Project http://adonthell.linuxgames.com
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY.
#
#  See the COPYING file for more details
#

import base
import adonthell

#
# This is class implements humanoid characters. Unlike others,
# they can equip and carry items. They also have a number of
# additional skills.
#
class humanoid (base.base):
    def __init__ (self, race):
        # -- init base class
        base.base.__init__ (self, race)
    
        # -- items the character has equipped
        self.__Equipment = adonthell.inventory ()
        
        # -- items the character carries
        self.__Backpack = adonthell.inventory (10)
        
        
    # -- equip count items in the given slot, if possible
    def equip (self, slot, item, count = 1):
        py_item = item.get_instance ()
        
        # -- get the slot the item should go to
        target = self.__Equipment.get_slot (slot)
        
        # -- does the character have such a slot? 
        if target == None: return 0
        
        # -- is the item made for the given slot?
        if slot not in getattr (py_item, "Slots", []): return 0

        # -- is the slot empty?
        if not target.accepts (item): return 0
        
        # -- does character meet item constraints?
        for (constraint, rank) in getattr (py_item, "Constraints", [])[:]:
            if getattr (self, constraint, 0) < rank: return 0

        # -- try to equip item(s)
        if target.add (item, count) == count: return 0
        
        # -- apply item effect
        item.equip (self.this)
        
        return 1
        
    
    # -- unequip count items from given slot, if possible
    def unequip (self, slot, item, count = 1):
        
        # -- get the slot the item should be removed from
        target = self.__Equipment.get_slot (slot)
        
        # -- does the character have such a slot? 
        if target == None: return 0
        
        # -- try to remove item(s)
        if target.remove (item, count) == 0: return 0
        
        # -- remove item effect
        item.unequip (self.this)
        
        return 1
