#
#  $Id$
#
#  (C) Copyright 2003 Kai Sterker <kaisterker@linuxgames.com>
#  Part of the Adonthell Project http://adonthell.linuxgames.com
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY.
#
#  See the COPYING file for more details
#

import damage

class chain (object):
    """A damage_chain is used to accumulate the damage a character does.
    
    It will collect all damage objects of the selected fighting feat and 
    equipped weapons, if any. Similar damage objects will be merged in
    the process.
    
    Damage chains are only valid for a single attack, as character stats
    or equipment may change in between."""

    def __init__ (self, character):
        """Create a new damage_chain.
        
        character: the character to examine."""
        self.__Chain = []
        
        # -- get effects of fighting feat
        feat = character.get_feat ()
        if feat != None: self.accumulate (feat.get_effects ())
        
        # -- get effects of weapon
        weapon = character.get_weapon ()
        if weapon != None: self.accumulate (feat.get_effects ())
        
        # -- fallback to unarmed combat
        else: self.accumulate (character.get_effects ())
        
        # -- sort damage objects by priority
        self.__Chain.sort ()
        
    def accumulate (self, d_list):
        """Helper method to add damage objects to damage chain."""
        for d_obj in d_list: 
            for obj in self.__Chain:
                if obj.merge (d_obj): continue
            self.__Chain.append (d_obj)

    def apply_effect (self, attacker, defender):
        """Apply the chain effects to attacker and/or defender.
        
        attacker: the character performing the attack.
        defender: the character being attacked."""
        for d_obj in self.__Chain:
            if d_obj.Type == damage.damage.SPECIAL:
                for obj in self.__Chain: d_obj.apply_effect (obj)
            else: d_obj.apply_effect (attacker, defender)            
    
    def to_string (self):
        """Return a textual description of the damage chain."""
        string = ""
        for d_obj in self.__Chain:
            string = "%s\n%s" % (d_obj.to_string (), string)
        return string
